#include <iostream>
#include <cxxopts.hpp>
#include <generate.h>

int main(int argc, char** argv)
{
	cxxopts::Options options("rug", "Random username generator");

	options.add_options()
		("min-char", "The minimum number of characters (including numbers)", cxxopts::value<short>()->default_value("8"))
		("max-char", "The maximum number of characters (including numbers)", cxxopts::value<short>()->default_value("10"))
		("min-num", "The minimum number of digits", cxxopts::value<short>()->default_value("0"))
		("max-num", "The maximum number of digits", cxxopts::value<short>()->default_value("4"))
		("count", "How many usernames should be generated", cxxopts::value<int>()->default_value("1"))
		("seed", "String used as seed sequence. If not defined, system time will be used instead.", cxxopts::value<std::string>()->default_value(""))
		("h,help", "Print help")
	;

	auto result = options.parse(argc, argv);

	if(result.count("help"))
	{
		std::cout << options.help() << std::endl;
		return 0;
	}

	short min_char = result["min-char"].as<short>();
	short max_char = result["max-char"].as<short>();
	short min_num = result["min-num"].as<short>();
	short max_num = result["max-num"].as<short>();
	int count = result["count"].as<int>();
	std::string seed(result["seed"].as<std::string>());

	if(min_char > max_char)
	{
		std::cerr << "ERROR: min-char cannot be greater than max-char" << std::endl << std::endl << options.help() << std::endl;
		return -1;
	}

	if(min_num > max_num)
	{
		std::cerr << "ERROR: min-num cannot be greater than max-num" << std::endl << std::endl << options.help() << std::endl;
		return -1;
	}

	if(max_num > min_char)
	{
		std::cerr << "ERROR: max-num cannot be greater than min-char" << std::endl << std::endl << options.help() << std::endl;
		return -1;
	}

	if(min_char < 0 || max_char < 0 || min_num < 0 || max_num < 0 || count < 0)
	{
		std::cerr << "ERROR: negative argument" << std::endl << std::endl << options.help() << std::endl;
		return -1;
	}

	generate(min_char, max_char, min_num, max_num, count, seed);
	return 0;
}
